#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# GTK
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
from gi.repository import GObject


# -------------------------------------------------------------------------
#
# This is used by plugins to create a menu of available filters
#
# -------------------------------------------------------------------------
def build_filter_model(space, local=[]):
    from gramps.gen.filters import CustomFilters

    model = Gtk.ListStore(GObject.TYPE_STRING, object)

    if isinstance(space, str):
        flist = local + CustomFilters.get_filters(space)
    elif isinstance(space, (list, tuple)):
        flist = space
    else:
        return None

    for filt in flist:
        model.append(row=[filt.get_name(), filt])
    return model
