/*
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2010 Rob G. Healey <robhealey1@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# $Id: $
#
*******************************************************************************
GRAMPS Cascading Style Sheet
Style Name: Combined Ancestor Tree Style Sheet
*******************************************************************************
#
===== Ancestor Graph Color Scheme =====
Males       #BCEAF6
Females     #FFC0CB
Unknown     #000
#
===== Web Graphics =====
Males       Web_Gender_Male.png
Females     Web_Gender_Female.png
# -------------------------------------------------------------------------- */
/* Subsections : Ancestors Tree -------------------------------------------- */
#tree {
    margin:0;
    padding:0;
    background:none;
    overflow-x:auto;
}
#toggle_anc {
    margin:0;
    padding:0;
    background:none;
    overflow-x:auto;
}
#treeContainer {
    position:relative;
    display: table-cell;
    vertical-align: middle;
    z-index:1;
}
#treeContainer div.boxbg {
    position:absolute;
    margin:0px;
    padding:0px;
    background:none;
}
#treeContainer div.boxbg a,
#treeContainer div.boxbg span.unlinked {
    position:relative;
    z-index:10;
    display:block;
    font:normal 1.1em/1.4em sans-serif;
    text-align:center;
    word-break:break-word;
    word-wrap:break-word;
    text-decoration:none;
    color: #00029D;
    width:280px;
    min-height: 70px;
    margin-left:16px;
    border: solid 2px #000;
    border-radius: 10px;
}
#treeContainer div.boxbg a.noThumb,
#treeContainer div.AncCol4 a,
#treeContainer div.AncCol4 span.unlinked {
    margin-top:10px;
    float: right;
}
#treeContainer div.boxbg a:hover {
    position:relative;
    z-index:999;
    font-size:1.3em;
    word-break:break-word;
    word-wrap:break-word;
    text-decoration:none;
    color: #00029D;
    width:400px;
    margin-left:-20px;
    padding: 0px 0px 0px 40px;
}
#treeContainer div.boxbg a:hover,
#treeContainer div.AncCol4 a:hover {
    margin-top:-44px;
}
#treeContainer div.boxbg a.noThumb:hover {
    margin-top:0;
    padding-left: 10px;
}
#treeContainer div.AncCol0 a:hover {
    margin-left:12px;
}
#treeContainer div.AncCol4 a:hover {
    margin-left:-60px;
}
#treeContainer div.boxbg span.thumbnail {
    display:block;
    position: absolute;
    max-width:85px;
    max-height:75px;
    left: 3px;
    top: 3px;
}
#treeContainer div.boxbg a.thumbnail table td.img {
    padding-right: 5px;
}
#treeContainer div.boxbg a.thumbnail:hover table td.img {
    padding-right: 9px;
}
#treeContainer div.boxbg a.thumbnail table td.name {
    padding-top:3px;
    padding-left: 2px;
    padding-right: 23px;
    width: 90%;
}
#treeContainer div.boxbg a.thumbnail img {
    margin-left:0px;
    padding-left: 0px;
    border-radius: 5px;
}
#treeContainer div.boxbg a.thumbnail:hover img {
    max-height:90%;
    margin-left:5px;
    position: absolute;
    left: 0px;
    top: 4px;
    border-radius: 5px;
}
#treeContainer div.boxbg table td:first-child, table th:first-child {
    padding-left: 5px;
    padding-top: 5px;
}
#IndividualDetail div.subsection table tr td:first-child {
    padding-left: 5px;
    padding-top: 5px;
}
#treeContainer #treeContainer div.AncCol4 span.thumbnail {
    display:none;
}
#treeContainer div.boxbg a:hover span.thumbnail {
    display:block;
}
#treeContainer div.boxbg span.fullname {
    display: none;
}
#treeContainer div.boxbg span.shortname {
    display: inline;
}
#treeContainer div.boxbg a:hover span.fullname {
    display: inline;
}
#treeContainer div.boxbg a:hover span.shortname {
    display: none;
}
#treeContainer div.boxbg a:hover {
    float: right;
    margin-top: 10px;
    border: solid 4px #000;
}
#treeContainer div.male a,
#treeContainer div.male span.unlinked {
    background:url(../images/Web_Gender_Male.png) #BCEAF6 no-repeat top right;
}
#treeContainer div.female a,
#treeContainer div.female span.unlinked {
    background:url(../images/Web_Gender_Female.png) #FFC0CB no-repeat top right;
}
#treeContainer div.other a,
#treeContainer div.other span.unlinked {
    background:url(../images/Web_Gender_Other.png) #94EF9E no-repeat top right;
}
#treeContainer div.unknown a,
#treeContainer div.unknown span.unlinked {
    background-color: #000;
    color: #FFF;
}
.shadow {
    display:none;
}
#tree div div.bvline {
    position:absolute;
    z-index:2;
    height:1px;
    margin:0 0 0 16px;
    padding:0;
    background-color: #00029D;
}
#tree div div.bhline {
    position:absolute;
    z-index:2;
    width:1px;
    margin:0 0 0 16px;
    padding:0;
    background-color: #00029D;
}
.ghline, .gvline {
    display:none;
}
