/*
# encoding: utf-8
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright 2012 Rob G. Healey <robhealey1@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

******************************************************
GRAMPS Cascading Style Sheet
Style Name: Web_Citations-Outline.css
******************************************************


    Source Citations Outline styles
------------------------------------------------------ */
div#SourceCitationReferents ul.Col1 {
    list-style-type: decimal
}
div#SourceCitationReferents ul.Col2 {
    list-style-type: upper-alpha
}
div#SourceCitationReferents ul.Col3 {
    list-style-type: lower-alpha
}
div#SourceCitationReferents ul.Col4 {
    list-style-type: upper-roman
}
div#SourceCitationReferents ul.Col5 {
    list-style-type: disc
}
